@@ -1,4 +1,6 @@
 import torch
+import logging
+
 import torch.nn as nn
 
 
@@ -66,6 +68,7 @@ class DiceLoss(nn.Module):
             Dice loss computed between :attr:`predicted` and :attr:`target`.
 
         """
+        logging.debug("Inside dice loss forward routine")
         predicted = predicted.float().view(-1)
         target = target.long().view(-1)
 
