@@ -1,5 +1,10 @@
 import numpy as np
 import torch
+import logging
+
+import torch.distributed as dist
+from torch.distributed import ReduceOp
+
 
 from medpy.metric.binary import (dc, jc, hd, asd, assd,
                                  precision, recall, ravd,
@@ -79,6 +84,7 @@ class Metrics():
                  phase='',
                  metrics_strings=['prec'],
                  num_classes=2,
+                 distributed=False,
                  *args, **kwargs):
         super(Metrics, self).__init__(*args, **kwargs)
         self.polyaxon_exp = polyaxon_exp
@@ -86,6 +92,8 @@ class Metrics():
         self.metric_strings = metrics_strings
         self.metric_funcs = {}
         self.metrics = {}
+        self.distributed = distributed
+
         self.multilabel = False
         self.num_classes = num_classes
         self.initialize_metrics()
@@ -125,6 +133,7 @@ class Metrics():
                                         Relative absolute volume difference.
 
         """
+        logging.debug("Initializing metrics")
         self.metrics['loss'] = []
 
         for metstr in self.metric_strings:
@@ -138,6 +147,7 @@ class Metrics():
         """Reset metrics map.
 
         """
+        logging.debug("Reset Metrics")
         for k in self.metrics.keys():
             self.metrics[k] = []
 
@@ -155,6 +165,7 @@ class Metrics():
             numpy array transformed from tensor.
 
         """
+        logging.debug("Inside _transform_tensor routine")
         if isinstance(tensor, int):
             return tensor
         if isinstance(tensor, torch.Tensor):
@@ -181,6 +192,7 @@ class Metrics():
             final tensor after applying transforms.
 
         """
+        logging.debug("Enter _argmax_or_thresholding routine")
         if type(tensor) != torch.Tensor:
             raise TypeError("Input should be a torch tensor!")
 
@@ -201,6 +213,7 @@ class Metrics():
                 tensor[tensor < 0.5] = 0
                 tensor[tensor >= 0.5] = 1
 
+        logging.debug("Exit _argmax_or_thresholding routine")
         return tensor
 
     def compute(self, predicted, target, loss):
@@ -226,16 +239,22 @@ class Metrics():
                 target = self._transform_tensor(target)
 
                 if 'mc' in k:
-                    self.metrics[k].append(self.metric_funcs[k](predicted.flatten(), target.flatten(),
+                    self.metrics[k].append(self.metric_funcs[k](predicted.flatten(),
+                                                                target.flatten(),
                                                                 average='micro'))
                 elif 'ml' in k:
-                    self.metrics[k].append(self.metric_funcs[k](predicted, target,
+                    self.metrics[k].append(self.metric_funcs[k](predicted,
+                                                                target,
                                                                 average='samples'))
                 elif 'mIoU' in k or 'SeK' in k:
-                    self.metrics[k].append(self.metric_funcs[k](predicted, target,
+                    self.metrics[k].append(self.metric_funcs[k](predicted,
+                                                                target,
                                                                 num_classes=self.num_classes))
                 else:
-                    self.metrics[k].append(self.metric_funcs[k](predicted, target))
+                    self.metrics[k].append(self.metric_funcs[k](predicted,
+                                                                target))
+
+                logging.info("{}={}".format(k, self.metrics[k][-1]))
 
     def crunch_it(self, epoch):
         """Crunch metrics to obtain mean_metrics map.
@@ -252,14 +271,41 @@ class Metrics():
 
         """
         mean_metrics = {}
+        mean_metrics_rank = {}
+
+        # compute
         for k in self.metrics.keys():
             if self.phase:
                 out_key = self.phase + '_' + k
             else:
                 out_key = k
+
             mean_metrics[out_key] = np.mean(self.metrics[k])
+            if self.distributed:
+                mean_metrics_rank[
+                    out_key + '_' + str(dist.get_rank())
+                ] = np.mean(self.metrics[k])
+
+        # all_reduce if phase != train
+        if self.distributed:
+            dist.barrier()
+            for k in mean_metrics.keys():
+                dist.all_reduce(torch.tensor(mean_metrics[k]).cuda(),
+                                op=ReduceOp.SUM)
+                mean_metrics[k] = mean_metrics[k].item() / float(dist.get_world_size())
+            dist.barrier()
+
+        # logging
+        for k in mean_metrics_rank.keys():
+            logging.info(f"rank_{k} = {mean_metrics_rank[k]}")
+        for k in mean_metrics.keys():
+            logging.info(f"mean_{k} = {mean_metrics[k]}")
 
         if self.polyaxon_exp:
-            self.polyaxon_exp.log_metrics(step=epoch, **mean_metrics)
-
+            if self.distributed:
+                if dist.get_rank() == 0:
+                    self.polyaxon_exp.log_metrics(step=epoch, **mean_metrics)
+                dist.barrier()
+            else:
+                self.polyaxon_exp.log_metrics(step=epoch, **mean_metrics)
         return mean_metrics
