@@ -1,6 +1,7 @@
 from torch.optim import (Adadelta, Adagrad, Adam, AdamW,
                          SparseAdam, Adamax, ASGD, LBFGS,
                          RMSprop, Rprop, SGD)
+import logging
 
 optimizer_map = {
     'adadelta': Adadelta,
@@ -35,6 +36,8 @@ def get_optimizer(key, args, model):
         optimizer instance.
 
     """
+    logging.debug("Enter get_optimizer routine")
     optimizer = optimizer_map[key]
     args['params'] = model.parameters()
+    logging.debug("Exit get_optimizer routine")
     return optimizer(**args)
