@@ -2,6 +2,10 @@ from .scheduler import get_scheduler
 from .optimizer import get_optimizer
 
 import torch
+import logging
+
+import torch.distributed as dist
+
 from torch.autograd import Variable
 from phobos.metrics.metrics import Metrics
 
@@ -38,7 +42,6 @@ class Runner():
         training epoch.
 
     """
-
     def __init__(self,
                  model,
                  criterion,
@@ -75,24 +78,52 @@ class Runner():
             self.scheduler = get_scheduler(args.scheduler, args.scheduler_args, self.optimizer)
         else:
             self.scheduler = None
+        self.distributed = False
+
+        if args.distributed:
+            self.set_distributed_params()
+
         self.train_metrics = Metrics(polyaxon_exp=polyaxon_exp,
                                      phase='train',
                                      metrics_strings=args.metrics,
-                                     num_classes=args.num_classes)
+                                     num_classes=args.num_classes,
+                                     distributed=self.distributed
+                                     )
         self.val_metrics = Metrics(polyaxon_exp=polyaxon_exp,
                                    phase='val',
                                    metrics_strings=args.metrics,
-                                   num_classes=args.num_classes)
+                                   num_classes=args.num_classes,
+                                   distributed=self.distributed
+                                   )
         self.epoch = 0
 
+    @staticmethod
+    def distributed():
+        """Initialize process group, default is nccl backend.
+
+        """
+        dist.init_process_group(backend='nccl')
+
+    def set_distributed_params(self):
+        """Set up distributed params: world size, rank, and distributed or not.
+
+        """
+        self.world_size = dist.get_world_size()
+        self.rank = dist.get_rank()
+        logging.info(f"{self.rank} / {self.world_size}")
+        self.distributed = True
+
     def set_epoch_metrics(self):
         """Sets epoch metrics and increments epoc count
             at the end of every epoch.
 
         """
+        logging.debug("Enter set_epoch_metrics routine")
         self.train_metrics.reset()
         self.val_metrics.reset()
         self.epoch += 1
+        logging.warning("Epoch : {}".format(self.epoch))
+        logging.debug("Exit set_epoch_metrics routine")
 
     def tensorize_batch(self, input_tensor, label_tensor):
         """Tensorize batch of input images and labels,
@@ -111,11 +142,13 @@ class Runner():
             input and label tensors in gpu.
 
         """
+        logging.debug("Enter tensorize_batch routine")
         input_tensor = Variable(input_tensor).float()
         label_tensor = Variable(label_tensor).long()
         if self.gpu > -1:
             input_tensor = input_tensor.to(self.gpu)
             label_tensor = label_tensor.to(self.gpu)
+        logging.debug("Exit tensorize_batch routine")
 
         return input_tensor, label_tensor
 
@@ -140,6 +173,7 @@ class Runner():
 
         """
         # Zero the gradient
+        logging.debug("Enter train_forward_backward routine")
         self.optimizer.zero_grad()
 
         # Get model predictions, calculate loss, backprop
@@ -147,6 +181,7 @@ class Runner():
         loss = self.criterion(prediction_tensor, label_tensor)
         loss.backward()
         self.optimizer.step()
+        logging.debug("Exit train_forward_backward routine")
 
         return prediction_tensor, loss
 
@@ -170,8 +205,11 @@ class Runner():
 
         """
         # Get predictions and calculate loss
+        logging.debug("Enter eval_forward routine")
         prediction_tensor = self.model(input_tensor)
         loss = self.criterion(prediction_tensor, label_tensor)
+        logging.debug("Exit eval_forward routine")
+
         return prediction_tensor, loss
 
     def train_model(self):
@@ -183,6 +221,7 @@ class Runner():
             dictionary of training metrics.
 
         """
+        logging.debug("Enter train_model routine")
         self.model.train()
 
         for input_tensor, label_tensor in self.train_loader:
@@ -195,6 +234,7 @@ class Runner():
 
             # clear batch variables from memory
             del input_tensor, label_tensor
+        logging.debug("Exit train_model routine")
 
         return self.train_metrics.crunch_it(self.epoch)
 
@@ -207,14 +247,15 @@ class Runner():
             dictionary of evaluation metrics.
 
         """
-        # self.model.eval()
+        self.model.eval()
+        logging.debug("Enter eval_model routine")
         with torch.no_grad():
-            for input_tensor, label_tensor in self.val_loader:
-                input_tensor, label_tensor = self.tensorize_batch(
-                    input_tensor, label_tensor)
+            for (input_tensor, label_tensor) in self.val_loader:
+                input_tensor, label_tensor = self.tensorize_batch(input_tensor, label_tensor)
 
                 prediction_tensor, loss = self.eval_forward(
                     input_tensor, label_tensor)
+
                 self.val_metrics.compute(prediction_tensor, label_tensor, loss)
 
                 # clear batch variables from memory
@@ -224,5 +265,7 @@ class Runner():
 
         if self.scheduler:
             self.scheduler.step(metrics['val_loss'])
+        logging.debug("Exit eval_model routine")
+        self.model.train()
 
         return metrics
