@@ -3,6 +3,8 @@ from torch.optim.lr_scheduler import (MultiplicativeLR, StepLR,
                                       ReduceLROnPlateau, CyclicLR,
                                       OneCycleLR, CosineAnnealingWarmRestarts)
 
+import logging
+
 scheduler_map = {
     'multiplicative': MultiplicativeLR,
     'step': StepLR,
@@ -34,7 +36,8 @@ def get_scheduler(key, args, optimizer):
         scheduler instance.
 
     """
-
+    logging.debug("Enter get_scheduler routine")
     scheduler = scheduler_map[key]
     args['optimizer'] = optimizer
+    logging.debug("Exit get_scheduler routine")
     return scheduler(**args)
